<?php
/**********************************************************************************************************************
CREWCOMMWIFI DIAGNOSTIC SCRIPT
WRITTEN BY JOSE MARI REYES

HISTORY:

4/15/2019
2/28/2020	FIX USING APICONFIG, RADUIS IP FIX

SERIES OF TEST IN ORDER:

1.  WAN INTERFACE TEST (UP | DOWN)
2.  GATEWAY TEST
3.  RADIUS UDP ACCEPT ACCEPT TEST
4.  SHIP SERVER PORT 10990 TEST 


***********************************************************************************************************************/
$force_fail = 0;


	if (isset($_SERVER['fail']) ==  true)  $force_fail = 1;
	

echo "<pre>";
echo "CREWCOMMWIFI DIAGNOSTIC (v 1.0)\n\n";
show_wan_ip_info();
echo "Analyzing ... please wait...\n";
flush();
echo "\n";
test_wan_interface();
echo "\n";
flush();
test_gateway();
echo "\n";
flush();
test_radius();
echo "\n";
flush();
test_server_10990();
echo "\n\n---Speed Test ----\n";
flush();
echo '</pre>';

echo '<div style="width:360px;text-align:right;"><iframe style="border:none;overflow:hidden !important;width:100%;height:360px;" src="//www.metercustom.net/plugin/?hl=en"></iframe></div>';


function test_server_10990()
{
ini_set("DEFAULT_SOCKET_TIMEOUT", 15);
global $force_fail;

	
	echo "--- Server Test ---\n";
	
	$port_80 = "http://93.157.3.167/api/testserver";
	$port_10990 = "http://93.157.3.167:10990/api/testserver";
	
	if ($force_fail == 1)
	{
		$port_80 = "http://93.157.3.16/api/testserver";
		$port_10990 = "http://93.157.3.16:10990/api/testserver";
	}
	
	$options = stream_context_create(array('http' => array('timeout' => 15)));
	
	
	
	echo "Trying port 80...\n"; flush();
	$ret = file_get_contents($port_80, false, $options);
	
	$v = strloc($ret, "Server=ShipServer");
	
	if ($v !== false)
		echo "Server port 80 : SUCCESS\n";
	else{
	
		echo "Server port 80 : FAILED\n";
		echo "Possible cause:\n";
		echo "- No Internet\n";
		echo "- Firewall Rules Blocking Port\n";                 
	}
	
	echo "Trying port 10990...\n"; flush();
	$ret = file_get_contents($port_10990, false, $options);

	$v = strloc($ret, "Server=ShipServer");
	
	if ($v !== false)
		echo "Server port 10990 : SUCCESS\n";
	else
	{
		echo "Server port 10990 : FAILED\n";
		echo "Possible cause:\n\n";
		echo "- No Internet\n";
		echo "- Firewall Rules Blocking Port\n";      
	}
}






function test_wan_interface()
{
include ('/www/cgi-bin/apiconfig');
	
	echo "--- WAN Test ---\n";
	echo "Reading Ethernet Ports (ENTERPRISE ONLY) WILL SHOW FAILED ON OTHER MACHINE";
	flush();

		
	if ($force_fail == 1) $eth = "dummy"; else $eth = WAN_ETH;
	
	$dump = shell_exec("ip addr | grep '$eth'");
	
	if (strpos($dump, "state UP") !== false)
		$ret = "SUCCESS";
	else
        $ret = "FAILED";

	echo "\nRESULT: $ret!\n\n";
	echo "$dump\n\n";
	
	if ($ret == "FAILED") 
	{
		echo "Check for:\n\n";
		echo "- Check Cable for break and continuity\n";
		echo "- Cable not properly connected/seated to the terminal or network port\n";
		echo "- Improper network configuration (check IP, Gateway, Broadcast and Subnet Parameters)\n\n";
	}
	
}


function test_radius()
{
global $dump;
global $force_fail;
  	
	$conf = get_rad_config();
	
    $conf = explode(":", $conf);
	$radserver = $conf[0];
	$radsecret = $conf[1];
	$radport   = 1812;

	
	if ($force_fail == 1) $radserver = "111.111.111.111";
	
	
	echo "--- RADIUS Test Server:$radserver ---\n";
	echo "Sending Accept-Accept Request... on UDP port: $radport\n";
	flush();
	
	
	$dump = shell_exec("/usr/bin/radtest joey joey $radserver $radport $radsecret");
	
	
	if (($loc = strpos($dump, "rad_recv:")) !== false)
	{
	 	$res = "RADIUS TEST: SUCCESS\n\n";
	 	$res .= substr($dump, $loc);// , strlen($dump));
		echo $res;
	} 
	else
	{
	 	echo "RADIUS TEST: FAILED.\n\n";
		echo "Possible Cause:\n\n";
		echo "- No Internet\n";
		echo "- Blocked RADIUS UDP port\n";
		echo "- RADIUS server unreachable\n";
		echo $dump;
	}


}




function test_gateway()
{

global $force_fail;

	echo "--- Gateway Test ---\n";
	$gt = get_gateway();

	echo "Contacting Terminal... (gateway ip:$gt)...\n";
	flush();
	
	
	if ($force_fail == 1) $gt = "111.111.111.111";

	$ret = shell_exec("ping -w 5 -c 1 $gt | grep 'packet'");
	
	echo "$ret\n";
	
	$p = strloc($ret, "0% packet loss");
	
	if ($p == true)
	{
		echo "Gateway Test: SUCCESS\n";
	}
	else
	{
		echo "Gateway Test: FAILED.  Unreachable Terminal\n\n";
		echo "Check for:\n\n";
		echo "- Bad/Broken Cable\n";
		echo "- Misconfigured Interface (IP, Gateway Broadcast, Subnet...)\n";
	}
	
	flush();
}






function get_rad_config()
{

	$v = shell_exec("uci show chilli.@chilli[0].radius");
	$v = explode("'", $v);
	$rad_ip = $v[1];
	
	$v = shell_exec("uci show chilli.@chilli[0].radsecret");
	$v = explode("'", $v);
	$rad_secret = $v[1];
	
	return "$rad_ip:$rad_secret";
}












function get_gateway()
{
include ('/www/cgi-bin/apiconfig');

        exec("netstat -nr | grep 'UG'", $tokens);
        //print_r($tokens);
	$wan_iface = WAN_ETH;

        for ($i = 0; $i < count($tokens); $i++)
        {
                if (strpos($tokens[$i], "$wan_iface") !== false)
                {
                        $v = explode(" ", $tokens[$i]);
                        return $v[9];
                        //print_r($v);
                }
        }
        return NULL;
}








function strloc($haystack, $needle)
{
        if ( preg_match("~\b$needle\b~",$haystack) )
                return true;
        else
                return false;

}




function show_wan_ip_info()
{
include ('/www/cgi-bin/apiconfig');

	$gt    = get_gateway();
	$if = WAN_ETH;
	
	$mac    = shell_exec("ifconfig $if | awk '/HWaddr/ { print $5 }'");
    $wan_ip = shell_exec("ifconfig $if | awk '/inet addr:/ { sub(/addr:/, \"\", $2); print $2 }'");
	
	$lan_ip = shell_exec("ifconfig tun0 | awk '/inet addr:/ { sub(/addr:/, \"\", $2); print $2 }'");
	
	echo "--- System info ---\n";
	echo "WAN IP  : $wan_ip";
	echo "GATEWAY : $gt\n";
	echo "MAC ADDR: $mac";
	echo "LAN IP  : $lan_ip\n";
	
	
}


?>
 
