<?php
require_once '/www/cgi-bin/firewall/config.php';
/*+
 * @author Tomex Evaristo DEC 2019
 * @SCRIPT_SYNC_VERSION 1
 * This script uses script sync version 1
 * do not modify since different sync versions output different data
 * this script might not able to process tha data
 */
define('SCRIPT_SYNC_VERSION','1');

/**
 * get current hash
 * this will return the current hash of the settings
 * server will check if router hash and server hash is the same
 * server will not return any data if both hashes are the same with server
 */
function get_hash()
{
    $SYNCED_DATA                = file_exists(SYNCAPPLY_INI) ? file_get_contents(SYNCAPPLY_INI) : '{}';
    $APPLIED_DATA               = file_exists(APPLIED_RULES) ? file_get_contents(APPLIED_RULES) : '{}';
    $BLOCKED_DOMAINS            = file_exists(BLOCKED_DOMAINS) ? file_get_contents(BLOCKED_DOMAINS) : '{}';
    $data_array                 = json_decode($SYNCED_DATA,true);
    $APPLIED_DATA_ARRAY         = json_decode($APPLIED_DATA,true);
    $BLOCKED_DOMAINS_ARRAY      = json_decode($BLOCKED_DOMAINS,true);


    if(isset($data_array['hash']) && isset($APPLIED_DATA_ARRAY['hash'])){
        $hash['config']             = $data_array['hash'];
    }else{
        $hash['config']             = 'ConfigHashNotSynced';
    }
    
    //hash for block os
    $HASH_CONFIG_BLOCK_OS 				= isset($BLOCKED_DOMAINS_ARRAY['hash_blckos']) ? $BLOCKED_DOMAINS_ARRAY['hash_blckos'] : 'HashNotFound1';
    $HASH_CONFIG_BLOCK_OS_APPLIED 		= isset($APPLIED_DATA_ARRAY['hash_blckos']) ? $APPLIED_DATA_ARRAY['hash_blckos'] : 'HashNotFound2';
	$hash['hash_blckos']                = $HASH_CONFIG_BLOCK_OS;
    $hash['hash_blckos_applied']        = $HASH_CONFIG_BLOCK_OS_APPLIED;
	return $hash;
}

function touch_applied_block_os_config()
{
    if(file_exists(CURRENT_APPLIED_CONFIG))
    {
        $data = file_get_contents(CURRENT_APPLIED_CONFIG);
        $data_parsed_array = json_decode($data,true);

        $data_parsed_array['CURRENT_APPLIED_BLOCK_OS'] = 'BLOCKOSNOTSYNCED';
        file_put_contents(CURRENT_APPLIED_CONFIG,$data_parsed_array);
    }
}

/**
 * Function wrapper for curl or file_get_contents
 * USES curl first if not found uses file_get_contents as fallback 
*/
function callAPI($url,$fields,$secure = true)
{
    $API_NONCE = '';
    $API_PASS  = '';
    if($secure)
    {
        function _generateToken($nonce)
        {		
            $TOKEN_SALT_1    = TOKEN_SALT_1;
            $TOKEN_SALT_2    = TOKEN_SALT_2;
            $prefix = substr(md5($nonce.$TOKEN_SALT_1.$nonce.$TOKEN_SALT_2) , 8, 17 );
            $suffix = substr(md5($TOKEN_SALT_1.$TOKEN_SALT_2.$nonce), 3 , 12);
            $sig    = substr( hash_hmac('sha256', $nonce , $TOKEN_SALT_1 ) , 17 );
            $sig    = substr( $sig , 0 , -12 );
            return $prefix.$sig.$suffix;
        }

        $API_NONCE = md5(rand(100,999));
        $API_PASS  = _generateTOken($API_NONCE);
    }

    if(function_exists('curl_init'))
    {
        $postdata = http_build_query($fields);
        //open connection
        $ch = curl_init();

        //set the url, number of POST vars, POST data
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, count($fields));
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postdata);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Secure-API-nonce:{$API_NONCE}",
            "Secure-API-pass:{$API_PASS}"			    
        ));

        //execute post
        $result = curl_exec($ch);
        $info = curl_getinfo($ch);
        //close connection
        curl_close($ch);

        $data = $result;
        $mode = 'CURL';

    }else
    {

        $mode = 'file_get_contents';
        $info = array();
        $postdata = http_build_query($fields);

        $opts = array('http' =>
            array(
                'method'  => 'POST',
                'header'  => array(
                    'Content-type: application/x-www-form-urlencoded',		
                    'User-Agent:API-SMSG',		
                    "Secure-API-nonce:{$API_NONCE}",
                    "Secure-API-pass:{$API_PASS}"			    		
                ),
                'content' => $postdata
            )
        );

        $context  = stream_context_create($opts);            
        
        try{
            $result = @file_get_contents($url, false, $context);    
            $info = @$http_response_header;
        }catch(Exception $e){

        }
                
        $data = $result;
    }

    if(LOG_CALLAPI)
    {
        $log['url'] 	     = $url;
        $log['fields'] 	     = $fields;
        $log['result']       = $data;
        $log['mode']         = $mode;
        $log['info']         = $info;
        $log['router_time']  = date('Y-m-d H:i:s');
        file_put_contents(CALLAPI_LOG,json_encode($log,JSON_PRETTY_PRINT));
    }		
    return $data;
}
 
function getIpMacParsedAsIni(){
    //Parse GetIpmac remove direct echo on buffer
    ob_start();
    require_once IPMAC_PATH;
    $ip_mac_data = ob_get_contents();
    ob_end_clean();
    //sanitize
    $sanitized_data = trim(strip_tags($ip_mac_data));
    //parse as ini
    $parsed_as_ini = parse_ini_string($sanitized_data);
    return $parsed_as_ini;
}   

function processSyncSettings($forceVesselUpdate)
{
    $_FORCEUPDATE = $forceVesselUpdate ? 'FORCED' : '';    

    $SRC = getIpMacParsedAsIni();
    $hashes = get_hash();
    $HW_INFO = hardwareInfo();
    if(is_cli())
    {
        $addedInfo = 'autosync'.'-'.$HW_INFO['hardware_status'];
    }else{
        $addedInfo = $_SERVER['REMOTE_ADDR'].'-'.$HW_INFO['hardware_status'];
    }
    $API_RESULT = callAPI( SYNC_URL ,array(
        'token' 		=> md5($SRC['mac']),
        'macaddress'  	=> $SRC['mac'],
        'addedinfo'    	=> $addedInfo,
        'hash'          => $hashes['config'].$_FORCEUPDATE,
        'hash_blckos'   => $hashes['hash_blckos'].$_FORCEUPDATE,
        'hardware'      => $HW_INFO['hardware_code'],
        'RouterSynced'  => SCRIPT_SYNC_VERSION,
    ),false);

    $RAW_CLEANED = trim($API_RESULT,'"');
    $API_RESULT_IS_PACKED = substr($RAW_CLEANED,0,6);
    if(strtoupper($API_RESULT_IS_PACKED) == 'PACKED')
    {
        //unpack and decode
        $API_RESULT_UNPACKED = substr($API_RESULT,8);
        if(function_exists('gzinflate'))
        {
            $inflated = gzinflate(base64_decode(strtr($API_RESULT_UNPACKED, '-_', '+/')));             
            $API_RESULT_PARSED['data'] = json_decode($inflated,true);

            if(is_array($API_RESULT_PARSED['data']) && count($API_RESULT_PARSED['data']) >= 1){
                $API_RESULT_PARSED['status']  = true;
                $API_RESULT_PARSED['message'] = 'Unpacked settings successfully';
            }else{
                $API_RESULT_PARSED['status']  = false;
                $API_RESULT_PARSED['message'] = 'Error unpacking settings';
            }            
        }else{
            $API_RESULT_PARSED = false;
            return array('status'=>false,'message'=>'Unable to unpack Settings. gzdeflate does not exist.');
        }        
    }else{
        $API_RESULT_PARSED = json_decode($API_RESULT,true);
    }
    
    saveBlockedDomains( $API_RESULT_PARSED['data'] , $hashes , $API_RESULT_PARSED['status'] );
    processBlockCrewUI( $API_RESULT_PARSED['data'] );
    saveAutoSyncConfig( $API_RESULT_PARSED );
    tagLastKnowServerAddr();
    
    if($API_RESULT_PARSED['status'] === true)
    {
        $_output['status'] = true;

        $IM_INI = array();
        $FW_INI = array();
        $ForIpPortHTML = array();
        $port = '';
        $cleanPorts = array();
        $cleanIPPorts = array();
        foreach($API_RESULT_PARSED['data'] as $API_K => $API_V)
        {
            if($API_K == 'crwapps')
            {
                $IM_INI = $API_V;
            }else{
                $FW_INI[$API_K] = $API_V;                
            }
        }

        $parsedPorts = array_filter(explode(';',$FW_INI['crwports']));

        foreach($parsedPorts as $parsedPortsData)
        {
            $dataPorts      = explode('|',$parsedPortsData);
            $PORTS          = $dataPorts[3];
            if(strpos( $PORTS , ',') !== false)
            {
                $port = prepportdata(array_filter(explode(',',$PORTS)),1);
            }else if(strpos($PORTS,':') !== false)
            {
                $port = prepportdata(array_filter(explode(':',$PORTS)),2);
            }else{
                $port = prepportdata($PORTS,3);
            }
           
            $cleanPorts[]   = $port;
        }

        $Iports = explode(';',$FW_INI['crwips']);
        foreach($Iports as $IPPORTS_)
        {
            $IpPorts_ = explode('|',$IPPORTS_);
            if(isset($IpPorts_[4]))
            {
                $PORTS    = $IpPorts_[4];
                if(strpos( $PORTS , ',') !== false)
                {
                    $port = prepportdata(array_filter(explode(',',$PORTS)),1);
                }else if(strpos($PORTS,':') !== false)
                {
                    $port = prepportdata(array_filter(explode(':',$PORTS)),2);
                }else{
                    $port = prepportdata($PORTS,3);
                }
            }

            if(count($IpPorts_) >= 4){
                $cleanIPPorts[] = _hex2ip($IpPorts_[2])."|".$port; 
            }
        }

        $PORTS_RAW      =   implode(";",$cleanPorts);
        $IPPORTS_RAW    =   implode(";",$cleanIPPorts);
        $FW_INI['rawips']   = $IPPORTS_RAW;
        $FW_INI['rawport']  = $PORTS_RAW;

        $API_RESULT_PARSED['data']['crwports']  = $PORTS_RAW;
        $API_RESULT_PARSED['data']['crwips']    = $IPPORTS_RAW;
        $FW_INI['router_time_updated'] = date('Y-m-d H:i:s e | c');


        $_output['message'][] = 'Server Message: '.$API_RESULT_PARSED['message'];
        $_output['dataSynced'] = $API_RESULT_PARSED['data'];
        
        if(ENABLE_ACFW)
        {
            file_put_contents(SYNCAPPLY_INI, json_encode($FW_INI) );
            file_put_contents(IMAPPS_INI, json_encode($IM_INI) );            

        }else{
            $_output['portforwarding'] = 'ACFW Disabled in config';
            $_output['firewall'] = 'ACFW Disabled in config';
        }

        $_output['message'][] = 'Router Message: Saved Data to INI files';
        $_output['message'][] = 'Router Message: Router needs to reboot to apply settings';

    }else{
        $IMAPPS = getAPPSasArray();
        // createAppsHtml($IMAPPS);
        
        $message = 'An error has been encountered, Unable to Connect to server';
        if(isset($API_RESULT_PARSED['message'])){
            $message = 'SERVER MESSAGE: '.$API_RESULT_PARSED['message'];
        }        
        $_output = array('status'=>false,'message'=>$message);
    }				

    return $_output;
}

function saveBlockedDomains($result_data,$current_hash,$status)
{    
	if(isset($result_data['BOS']) && $result_data['BOS'] == 1)
	{
		if(is_array($result_data) && isset($result_data['hash_blckos']) && isset($result_data['crwblockos']))
		{
			//Disabled
			if($result_data['hash_blckos']  == $current_hash['hash_blckos_applied'])
			{
				//Do Nothing
			}else
			{
		        $write['crwblockos']    = isset($result_data['crwblockos']) ? $result_data['crwblockos'] : '';
		        $write['hash_blckos']   = isset($result_data['hash_blckos']) ? $result_data['hash_blckos'] : '';
		                   
	            file_put_contents(BLOCKED_DOMAINS,json_encode($write));
		    }
		}
	}
}

function getAPPSasArray()
{
    $IMAPPS         = file_get_contents(IMAPPS_INI);
    $IMAPPS_DATA    = json_decode($IMAPPS,TRUE);
    return $IMAPPS_DATA;
}

function hardwareInfo()
{
	//returns hardware type    
    if(file_exists(API_CONFIG_FILE_PATH))
    {            
        //Hardware Types config file, so that we can update sync dynamically
        include API_CONFIG_FILE_PATH;
        if(!defined('MACHINE')){
            $CODE       = '#4041';
            $MACHINE    = 'OV';    
        }else{
            $CODE       = '#200';
            $MACHINE    = MACHINE;     
        }       
    }else{
        $CODE       = '#404';
        $MACHINE    = 'OV';
    }

    $hw_info['hardware_code']  = $MACHINE;
    $hw_info['hardware_status']   = $CODE;
    return $hw_info;
}

function loadcurrentfwsettings()
{
    $firewall       = '<br><br><strong>Firewall:</strong><br>';
    $portforward    = '<br><br><strong>Portforward Rules:</strong><br>';
    $dnsmasq        = '<br><br><strong>dnsmasq.conf:</strong><br><br>';
    if(file_exists(FIREWALL_CONF_FILE)){
        
        $lines = file(FIREWALL_CONF_FILE);

        foreach($lines as $line)
        {
            $firewall .= $line."<br>";
        }

    }else{
        $firewall .= 'Unable to locate firewall file';
    }

    if(file_exists(FORWARD_CONF__FILE)){
        
        $pflines = file(FORWARD_CONF__FILE);

        foreach($pflines as $pfline)
        {
            $portforward .= $pfline."<br>";
        }
    }else{
        $portforward .= 'Unable to locate firewall file';
    }
    

    if(file_exists('/etc/dnsmasq.conf'))
    {
        $confline = file('/etc/dnsmasq.conf');
        foreach($confline as $dnsmasqline)
        {
            $dnsmasq .= $dnsmasqline."<br>";
        }
    }else{
        $dnsmasq .= "dnsmasq file not found";
    }

    $data = array();
    $data['firewall']       = $firewall;
    $data['portforward']    = $portforward;
    $data['dnsmasq']        = $dnsmasq;
    $data['status']         = true;
    $data['message']        = 'Firewall and Portforward rules fetched';
    return $data;
}

function _hex2ip($hex)
{
    if($hex == 0 || $hex == "0"){
        return 0;
    }
    $tmp = $hex[0] . $hex[1];
    $ip = hexdec($tmp) . ".";
    
    $tmp = $hex[2] . $hex[3];
    $ip .= hexdec($tmp) . ".";
    
    $tmp = $hex[4] . $hex[5];
    $ip .= hexdec($tmp) . ".";
    
    $tmp = $hex[6] . $hex[7];
    $ip .= hexdec($tmp);
    
    if (strlen($hex) > 8)  $ip .= substr($hex, 8);
    
    return str_replace(' ', '', $ip);
}

function prepportdata($data,$mode = 0)
{    
    if($data == 0){
        return "";
    }

    if( $mode == 1)
    {
        foreach ($data as $portsK => $portsV) {
            $tmp[] = hexdec($portsV);           
        }
        $parsed = implode(',',$tmp);

    }else if($mode == 2)
    {
        $params = $data;
        $parsed = hexdec($params[0]).":".hexdec($params[1]);
    }else{ 
        $parsed = hexdec($data);
    }
    
    return $parsed;
}

function _replaceConfig($main = array(),$replace = array())
{
	return array_replace($main,$replace);
}

function updateWANsettings($params)
{
    function _internalApplyChanges($x_protocol = '',$x_ipaddress = '',$x_netmask = '',$x_gatway = '',$x_broadcast = '',$x_dns = '8.8.8.8'){

        $textBuild = implode(';',array($x_protocol,$x_ipaddress,$x_netmask,$x_gatway,$x_broadcast));
        ob_start();
        require_once NETWORK_PATH;    
        if(strtoupper($x_protocol) == 'DHCP'){
            set_wan('DHCP');
        }else{
            set_wan($x_protocol, $x_ipaddress, $x_netmask, $x_gatway, $x_broadcast, $x_dns );                
        }        

        file_put_contents('/www/cgi-bin/firewall/log/setwan.log',"set_wan($x_protocol, $x_ipaddress, $x_netmask, $x_gatway, $x_broadcast, $x_dns );");
        $result = ob_get_contents();
        ob_end_clean();

        $SRC = getIpMacParsedAsIni();
        $API_RESULT = callAPI( UPDATE_URL ,array(
            'token' => md5($SRC['mac']),
            'macaddress' => $SRC['mac'],
            'method' => 'wan',
            'wan_protocol' => $x_protocol,
            'wan_build' => $textBuild
        ),false);
    }

	$current_settings = file_get_contents(APPLIED_RULES);
	
	$parsed = json_decode($current_settings,true);
    $message = array();

    $curr_wan_settings = array_filter(explode(';',$parsed['crwwansett']));
    $curr_dns_settings = array_filter(explode(';',$parsed['crwwandns']));
    
    if(isset($params['wan_mode']))
    {
        switch ( strtoupper($params['wan_mode'])) {
            case 'DHCP':
                $status = true;

                if($curr_wan_settings[0] == 'DHCP')
                {
                	$message[] = 'Selected WAN Mode is already in DHCP';
                }else{
	                $message[] = 'Changes was saved, Updated to DHCP. Router will reboot';
	                // set_wan('dhcp');	
	                $final = _replaceConfig($parsed,array('crwwansett'=>'DHCP;'));
	                file_put_contents(APPLIED_RULES,json_encode($final));
                    _internalApplyChanges('DHCP');
                    
                    sleep(5);
                    exec('reboot');
                }
                
            break;
            case 'STATIC':
                $proceed = true;
                
                if(!isset($params['wan_ip']) || !ip2long($params['wan_ip']))
                {
                    $message[] = 'Invalid IP Address'; 
                    $proceed = false;
                }

                if(!isset($params['wan_subnet']) || !ip2long($params['wan_subnet']))
                {
                    $message[] = 'Invalid Subnet Mask'; 
                    $proceed = false;
                }

                if(!isset($params['wan_gateway']) || !ip2long($params['wan_gateway']))
                {
                    $message[] = 'Gateway Address not valid'; 
                    $proceed = false;
                }

                if(!isset($params['wan_broadcast']) || !ip2long($params['wan_broadcast']))
                {
                    $message[] = 'Broadcast Address not valid'; 
                    $proceed = false;
                }

				if( $params['wan_ip'] == @$curr_wan_settings[1] &&
					$params['wan_subnet'] == @$curr_wan_settings[2] &&
					$params['wan_gateway'] == @$curr_wan_settings[3] &&
					$params['wan_broadcast'] == @$curr_wan_settings[4])
				{                    
                    $message[] = 'No changes were made on the IP.'; 
                    $proceed = false;
				}

                if($proceed){
                    $status = true;
                    // set_wan
                    //updated applied_rules.hjson for new settings                    
                    $build = "STATIC;{$params['wan_ip']};{$params['wan_subnet']};{$params['wan_gateway']};{$params['wan_broadcast']}";
                    $final = _replaceConfig($parsed,array('crwwansett'=>$build));
                    file_put_contents(APPLIED_RULES,json_encode($final));                    
                    _internalApplyChanges('STATIC',$params['wan_ip'],$params['wan_subnet'],$params['wan_gateway'],$params['wan_broadcast']);                
                    $message[] = 'Changes was saved, Updated to STATIC .Router will reboot';

                    sleep(5);
                    exec('reboot');

                }else{
                    $status = false;
                }
            break;
            default:
                // code...
                $status = false;
                $message[] = 'Unable to process Request';
            break;          
        }
    }else{
        $status = false;
        $message[] = 'Unable to process WAN mode';
    }

    return array(
        'status' => $status,
        'message' => implode('<br>',$message)
    );
}

function processBlockCrewUI($data)
{    
    if(isset($data['crwui']))
    {           
        @file_put_contents(CREW_UI_DISABLED_PATH,$data['crwui']);
    }
}

function is_cli() {
    return (!isset($_SERVER['SERVER_SOFTWARE']) && (php_sapi_name() == 'cli' || (is_numeric($_SERVER['argc']) && $_SERVER['argc'] > 0)));
}

function saveAutoSyncConfig($params){  
    if(is_array($params) && count($params >= 1) && isset($params['syncset']))
    {
        $time_command = trim($params['syncset']);
        $croncmds = array();
        /*
            CronJob In order
            SYNCFW
            APPLYFW
            SYNC_SETTINGS
        */
        $croncmds[] = $time_command.'|/usr/bin/php-fcgi /www/cgi-bin/event.synchronize.settings.php SYNC_SETTINGS > /www/crontestoutput.txt';
        $croncmds[] = $time_command.'|/usr/bin/php-fcgi /www/cgi-bin/event.synchronize.settings.php SYNCFW >> /www/crontestoutput.txt';
        $croncmds[] = $time_command.'|/usr/bin/php-fcgi /www/cgi-bin/event.synchronize.settings.php APPLYFW >> /www/crontestoutput.txt';
		if(file_exists(JOBSCHED_PATH)){
			ob_start();
            require_once JOBSCHED_PATH;  

            if($time_command == "" || strlen($time_command) == 0){
                macros_process('');
            }else{
                $finalCommand = implode(';',$croncmds);            
                macros_process($finalCommand);
            }
			ob_end_clean();
		}        
    }
}

function tagLastKnowServerAddr()
{
    if(isset($_SERVER['SERVER_ADDR']))
    {
        file_put_contents(LAST_KNOWN_SERVER_ADDR,trim($_SERVER['SERVER_ADDR']));
    }
}