<?php
/*
	Syncsettings
	vessel config
	vessel settings
	and update vesconf.txt
	all excerpts from the old sourcecode move into a new file to call sync modular
	@author tomex Evaristo
*/
class Syncsettings{

	const PORTACCESS = 10990;
	const IPMACFILE = '/www/cgi-bin/getipmac.php';
	const IPFILE1	= "/www/version.txt"; //Old version file
	const IPFILE2	= "/www/version.ini"; //new version file
	const SSETTINGS = 'smsglobal/api/rest/requestfleetonesettings';
	const VSETTINGS = 'smsglobal/api/rest/requestvesselinfo';
	const VCONFIG   = 'smsglobal/api/rest/getshipconfig';
	const DATAPATHS = '/www/localstore/';
	const DATAFILE  = '/www/vesconf.txt';
	public $api_key = '1234567890';
	private $HARDWARE_MACADDRESS = false;
	private $SYNC_RESULT = array(0,0,0,0,0,0);
	private $_DEBUG_VARS = array();
	const DEFAULT_SHIP_IP = '93.157.3.167';
	const SHIP_IP_ADDRESS = '';

	//error threshold
	//if returned data is shorter than SYNC_ERROR_THRESHOLD value then this is an error
	const SYNC_ERROR_THRESHOLD = 5;
	
	public function __construct()
	{
		$this->_DEBUG_VARS[] = __CLASS__." has been initialized";
		$this->getIPandMac();
		$this->SHIP_IP_ADDRESS = $this->getServerIP();
	}

	public function buildURL($URL = ''){
		$IP 	= $this->SHIP_IP_ADDRESS;
		$PORT 	= self::PORTACCESS;
		return "http://{$IP}:{$PORT}/$URL";
	}

	public function macaddress($suppliedMac){
		$this->HARDWARE_MACADDRESS = $suppliedMac;
	}

	private function getServerIP()
	{		
		if(file_exists(self::IPFILE1))
		{			
			$data 	= file_get_contents(self::IPFILE1);
			$parsed = str_replace(':', '=', $data);
			$array  = parse_ini_string($parsed);			
			$IP =  $array['SSIP'];
			$log = "Using ip file 1 {$IP}";

		}else if(file_exists(self::IPFILE2))
		{
			$data = @parse_ini_file(self::IPFILE2);
			$IP_arr = $data['shipserverip'];
			$IP = $IP_arr[0];
			$log = "Using ip file 2 {$IP}";
		}else{			
			$IP = self::DEFAULT_SHIP_IP;
			$log = "Unable to locate IP file 1 and 2, using {$IP}";
		}

		$this->_DEBUG_VARS[][__FUNCTION__] = $log;
		return $IP;
	}

	public function getIPandMac()
	{
		$this->_DEBUG_VARS[] = 'Retreiving MacAddress';
        ob_start();
        include_once self::IPMACFILE;
        $ip_mac_data = ob_get_contents();
        ob_end_clean();
        //sanitize
        $sanitized_data = trim(strip_tags($ip_mac_data));
        //parse as ini
        $parsed_as_ini = parse_ini_string($sanitized_data);         
        $this->HARDWARE_MACADDRESS =  isset($parsed_as_ini['mac']) ? $parsed_as_ini['mac'] : '';
        $this->_DEBUG_VARS[] = 'Retreiving MacAddress - Got : '.$this->HARDWARE_MACADDRESS;
        return $this;
	}

	/**
	 * Function wrapper for curl or file_get_contents
	 * USES curl first if not found uses file_get_contents as fallback 
	*/
	private function callAPI($url,$fields)
	{

		$dateNow = date('Y-m-d H:i:s');
	    if(function_exists('curl_init'))
	    {
	        $postdata = http_build_query($fields);
	        //open connection
	        $ch = curl_init();

	        //set the url, number of POST vars, POST data
	        curl_setopt($ch, CURLOPT_URL, $url);
	        curl_setopt($ch, CURLOPT_POST, count($fields));
	        curl_setopt($ch, CURLOPT_POSTFIELDS, $postdata);
	        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
	            "X-SMSG-SYNC-VSETTINGS:{$dateNow}",	            		   
	        ));

	        //execute post
	        $result = curl_exec($ch);
	        $info = curl_getinfo($ch);
	        //close connection
	        curl_close($ch);

	        $data = $result;
	        $mode = 'CURL';

	    }else
	    {
	        $mode = 'file_get_contents';
	        $info = array();
	        $postdata = http_build_query($fields);

	        $opts = array('http' =>
	            array(
	                'method'  => 'POST',
	                'header'  => array(
	                    'Content-type: application/x-www-form-urlencoded',		
	                    'User-Agent:API-SMSG',		
	                    "X-SMSG-SYNC-VSETTINGS:{$dateNow}",	                    			    	
	                ),
	                'content' => $postdata
	            )
	        );

	        $context  = stream_context_create($opts);            	       
	        $result = @file_get_contents($url, false, $context);
	        $data = $result;
	    }

		$this->_DEBUG_VARS[] = "Processing $url using $mode";
	    return $data;
	}	


	public function process_settings()
	{
		$url 		= $this->buildURL(self::SSETTINGS);
		$params 	= array('macid'=>$this->HARDWARE_MACADDRESS );
		$response 	= $this->callAPI( $url , $params );

		$this->_DEBUG_VARS[__FUNCTION__]['URL'] 			= $url;
		$this->_DEBUG_VARS[__FUNCTION__]['PARAMETERS'] 	= $params ;
		$this->_DEBUG_VARS[__FUNCTION__]['RESPONSE'] 		= 'ERROR NO RESPONSE WAS RECIEVED' ;
		if(strlen($response) > self::SYNC_ERROR_THRESHOLD)
		{
			$this->_DEBUG_VARS[__FUNCTION__]['RESPONSE'] = $response;
			$this->SYNC_RESULT[0] = 1;
			//Save
			if( file_put_contents(self::DATAFILE,date("Ymd") . "[DSEP]" . $response) )
			{
				$this->SYNC_RESULT[3] = 1;
			}
		}
		
		return $this;
	}

	public function process_vsettings()
	{

		$DataFileName = base64_encode("getVesselSettings");
		$params = array(
			'macid'		=> $this->HARDWARE_MACADDRESS,
			'api_key'	=> $this->api_key,
			'log'		=> '9b59ea58d5c5467-000000',
			'cssid'		=> '0158078309500000001'
		);		
		$url = $this->buildURL(self::VSETTINGS);
		$response = $this->callAPI($url,$params);

		$this->_DEBUG_VARS[__FUNCTION__]['URL'] 			= $url;
		$this->_DEBUG_VARS[__FUNCTION__]['PARAMETERS'] 	= $params ;
		$this->_DEBUG_VARS[__FUNCTION__]['RESPONSE'] 		= 'ERROR NO RESPONSE WAS RECIEVED' ;
		if(strlen($response) > self::SYNC_ERROR_THRESHOLD)
		{
			$this->_DEBUG_VARS[__FUNCTION__]['RESPONSE'] = $response ;
			$this->SYNC_RESULT[1] = 1;
			//Save
			if(file_put_contents(self::DATAPATHS.'/'.$DataFileName , base64_encode($response) ) )
			{
				$this->SYNC_RESULT[4] = 1;
			}
		}
		return $this;
	}

	public function process_vconfig()
	{
		$DataFileName = base64_encode("getVesselConfig");
		$url = $this->buildURL(self::VCONFIG);
		$params = array(
			'macid'		=> $this->HARDWARE_MACADDRESS,
			'api_key'	=> $this->api_key,
			'log'		=> '9b59ea58d5c5467-000000',
			'cssid'		=> '0158078309500000001'
		);		
		// $response = $this->callAPI($url,$params);
		$final_url_with_data = $url."?".http_build_query($params);
		$response = file_get_contents($final_url_with_data);

		$this->_DEBUG_VARS[__FUNCTION__]['URL'] 			= $final_url_with_data;
		$this->_DEBUG_VARS[__FUNCTION__]['PARAMETERS'] 	= $params ;
		$this->_DEBUG_VARS[__FUNCTION__]['RESPONSE'] 		= 'ERROR NO RESPONSE WAS RECIEVED' ;

		if(strlen($response) > self::SYNC_ERROR_THRESHOLD)
		{
			$this->SYNC_RESULT[2] = 1;
			//Save
			$this->_DEBUG_VARS[__FUNCTION__]['RESPONSE'] 		= $response;
			if(file_put_contents(self::DATAPATHS.'/'.$DataFileName , base64_encode($response) ) )
			{					
				$this->SYNC_RESULT[5] = 1;
			}
		}		
		return $this;
	}

	public function sync_result(){
		return implode('',$this->SYNC_RESULT);
	}

	public function _output_debug_result(){
		$this->_DEBUG_VARS[]['SYNC_RESULT'] = $this->sync_result();
		return $this->_DEBUG_VARS;
	}
}


//#USE CASE BELLOW

/* Create Class Instance
 * This will auto retrieve Server IP base on the version.txt and version.ini
 * This will auto retrieve MAC ADDRESS	
*/
// $sync = new Syncsettings;


/*
 * @void
 * Request First sync, This data holds the basic firewall rules
 * also the time,data,UI version,monthly and daily time/data reset
*/
// $sync->process_settings();

/*
 * @void
 * Sync Vessel Settings
 * API_KEY is optional by changing this we can tell in our serverlogs
 * if sync was done via Dashboard or the cronjob
*/
// $sync->api_key = 'AutoSyncDataVsettings';
// $sync->process_vsettings();

/*
 * @void
 * Sync Vessel Config
 * API_KEY is optional by changing this we can tell in our serverlogs
 * if sync was done via Dashboard or the cronjob
*/
// $sync->api_key = 'AutoSyncDataVConfig';
// $sync->process_vconfig();


/*
 * Returns string 000000
 * index[1] refers to process_settings 1 server returns data , 0 no data was returned
 * index[2] refers to process_vsettings 1 server returns data , 0 no data was returned
 * index[3] refers to process_vconfig 1 server returns data , 0 no data was returned
 * index[4] refers to process_settings saving config file 1 data was successfully written , 0 failed
 * index[5] refers to process_vsettings saving config file 1 data was successfully written , 0 failed
 * index[6] refers to process_vconfig saving config file 1 data was successfully written , 0 failed
*/
// echo $sync->sync_result();
